@@ -1,21 +1,20 @@
 """Package protogen makes writing protoc plugins easy.
 
-A protoc plugin essentialy turns a CodeGeneratorRequest from protoc into
-CodeGeneratorResponse. The CodeGeneratorRequest contains the raw proto
-descriptors of the proto definitions contained in the files code generation is
-requested for (and the descriptors of every file thats imported). The
-CodeGeneratorResponse is returned by to plugin to protoc. It contains a list of
-files (name and content) the plugin wants protoc to write to disk.
+Working with the raw protobuf descriptor messages can be cumbersome.  `protogen`
+resolves and links the dependencies and references between the raw Protobuf
+descriptors and turns them into their corresponding `protogen` classes that are
+easier to work with.  It also provides mechanisms that are espacially useful to
+generate Python code like dealing with Python imports.
 
-``protogen`` provides a bunch of classes to ease writing protoc plugins. Most of
-them are simply replacements of their corresponding descriptors. E.g.
-:class:`File` represents a proto FileDescriptor, :class:`Message` a proto
-Descriptor, :class:`Service` a proto ServiceDescriptor etc. They should be self
-explanatory. You can read their docstrings for more information about them.
+Most classes in `protogen` are simply replacements of their corresponding
+Protobuf descriptors: `protogen.File` represents a FileDescriptor,
+`protogen.Message` a Descriptor, `protogen.Field` a FieldDescriptor and so on.
+They should be self explanatory. You can read their docstrings for more
+information about them.
 
-The classes :class:`Options`, :class:`Plugin` and :class:`GeneratedFile` make up
-a framework to generate (Python) files from a CodeGeneratorRequest.  You can see
-these in action in the following example plugin:
+The classes `protogen.Options`, `protogen.Plugin` and `protogen.GeneratedFile`
+make up a framework to generate files.  You can see these in action in the
+following example plugin:
 
 .. code-block:: python
 
@@ -36,12 +35,10 @@ these in action in the following example plugin:
             g.P()
             for m in f.message:
                 g.P("class ", m.py_ident, ":")
-                g.P("  pass")
                 for ff in m.fields:
                     # ...
             for s in f.services:
                 g.P("class ", s.py_ident, ":")
-                g.P("  pass")
                 for m in f.methods:
                     g.P("  def ", m.py_name, "(request):")
                     g.P("    pass")
@@ -823,8 +820,8 @@ class Field:
             if self.message is None:
                 raise ResolutionError(
                     file=self.parent_file.proto.name,
-                    ref=self.full_name,
-                    type_name=self.proto.type_name,
+                    desc=self.full_name,
+                    ref=self.proto.type_name,
                 )
 
 
@@ -1205,14 +1202,11 @@ class File:
         self,
         proto: google.protobuf.descriptor_pb2.FileDescriptorProto,
         generate: bool,
-        py_import_func: Callable[[str, str], str],
+        py_import_func: Callable[[str, str], PyImportPath],
     ):
         self.proto = proto
         self.generated_filename_prefix = proto.name[: -len(".proto")]
-        # The actual pyton path is determined using the py_import_func.
-        # TODO maybe require the rewrite func to return a Python file and
-        # then generate the import path from that python filename
-        self.py_import_path = PyImportPath(py_import_func(proto.name, proto.package))
+        self.py_import_path = py_import_func(proto.name, proto.package)
         self.py_package_name = str(proto.package)
         self.generate = generate
         self.dependencies: List[File] = []
@@ -1467,15 +1461,19 @@ class Plugin:
     files_to_generate : List[File]
         Set of files to code generation is request for. These are the files
         explictly passed to protoc as command line arguments.
+    registry : Registry
+        The registry that was used in the resolution process for this plugin.
     """
 
     def __init__(
         self,
         parameter: Dict[str, str],
         files_to_generate: List[File],
+        registry: Registry,
     ):
         self.parameter = parameter
         self.files_to_generate = files_to_generate
+        self.registry = registry
 
         self._error: Optional[str] = None
         self._generated_files: List[GeneratedFile] = []
@@ -1532,7 +1530,7 @@ class Plugin:
             self._error = msg
 
 
-def default_py_import_func(filename: str, package: str) -> str:
+def default_py_import_func(filename: str, package: str) -> PyImportPath:
     """Return the Python import path for a file.
 
     Return the Python import path for a file following the behaviour of the
@@ -1550,7 +1548,7 @@ def default_py_import_func(filename: str, package: str) -> str:
 
     Returns
     -------
-    str
+    PyImportPath
         The Python import path for the file.
 
     Example
@@ -1558,7 +1556,7 @@ def default_py_import_func(filename: str, package: str) -> str:
     >>> default_py_import_func("google/protobuf/field_mask.proto", "google.protobuf")
     "google.protobuf.field_mask_pb2"
     """
-    return filename.replace(".proto", "_pb2").replace("/", ".")
+    return PyImportPath(filename.replace(".proto", "_pb2").replace("/", "."))
 
 
 class Options:
@@ -1575,7 +1573,7 @@ class Options:
     def __init__(
         self,
         *,
-        py_import_func: Callable[[str, str], str] = default_py_import_func,
+        py_import_func: Callable[[str, str], PyImportPath] = default_py_import_func,
         input: BinaryIO = sys.stdin.buffer,
         output: BinaryIO = sys.stdout.buffer,
     ):
@@ -1583,7 +1581,7 @@ class Options:
 
         Arguments
         ---------
-        py_import_func : Callable[[str, str], str], optional
+        py_import_func : Callable[[str, str], PyImportPath], optional
             Defines how to derive :class:`PyImportPath` for the :class:`File`
             objects in the resolution process. This also influences the
             :class:`PyIdent` attributes that are part of :class:`Message`,
@@ -1671,7 +1669,7 @@ class Options:
                 files_to_generate.append(file)
 
         # Create plugin and run the provided code generation function.
-        plugin = Plugin(parameter, files_to_generate)
+        plugin = Plugin(parameter, files_to_generate, registry)
         f(plugin)
 
         # Write response.
